/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Nde - a lightweight, Qt based, desktop toolset
 * https://lxqt.org
 *
 * Copyright: 2010-2011 Razor team
 * Authors:
 *   Petr Vanek <petr@scribus.info>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include <Nde/SingleApplication>
#include <Nde/Translator>
#include <QDesktopWidget>
#include <QCommandLineParser>
#include <QCoreApplication>
#include "leavedialog.h"
#include <QDebug>
#include "leavewidget.h"
#include <kwindowsystem.h>
#include <QDialog>
#include <QScreen>
#include <Nde/PowerManager>

#define NDE_SERVICE      "org.nde.session"
#define NDE_PATH         "/NdeSession"
#define NDE_INTERFACE    "org.nde.session"

int main(int argc, char *argv[])
{
    Nde::SingleApplication a(argc, argv);
    a.setAttribute(Qt::AA_UseHighDpiPixmaps, true);

    Nde::Translator::translateApplication();
    QCommandLineParser parser;
    parser.setApplicationDescription(QStringLiteral("Nde Leave"));
    const QString VERINFO = QStringLiteral(NDE_SESSION_VERSION
                                           "\nlibnde   " NDE_VERSION
                                           "\nQt        " QT_VERSION_STR);
    a.setApplicationVersion(VERINFO);

    parser.addHelpOption();
    parser.addVersionOption();

    QCommandLineOption logoutOption(QStringLiteral("logout"), QCoreApplication::translate("main", "Logout."));
    parser.addOption(logoutOption);

    QCommandLineOption lockscreenOption(QStringLiteral("lockscreen"), QCoreApplication::translate("main", "Lockscreen."));
    parser.addOption(lockscreenOption);

    QCommandLineOption suspendOption(QStringLiteral("suspend"), QCoreApplication::translate("main", "Suspend."));
    parser.addOption(suspendOption);

    QCommandLineOption hibernateOption(QStringLiteral("hibernate"), QCoreApplication::translate("main", "Hibernate."));
    parser.addOption(hibernateOption);

    QCommandLineOption shutdownOption(QStringLiteral("shutdown"), QCoreApplication::translate("main", "Shutdown."));
    parser.addOption(shutdownOption);

    QCommandLineOption rebootOption(QStringLiteral("reboot"), QCoreApplication::translate("main", "Reboot."));
    parser.addOption(rebootOption);

    QCommandLineOption switchUserOption(QStringLiteral("switchUser"), QCoreApplication::translate("main", "switchUser."));
    parser.addOption(switchUserOption);

    parser.process(a);

    if (parser.isSet(switchUserOption)) {
        Nde::PowerManager powermanager(&a);
        powermanager.switchuser();
        return 0;
     }else if (parser.isSet(hibernateOption)){
        Nde::PowerManager powermanager(&a);
        powermanager.hibernate();
        return 0;
    }

    LeaveDialog *leaveDialog = new LeaveDialog();

    if (parser.isSet(logoutOption)){
        leaveDialog->setConfirmText(LeaveWidget::Logout);
    }else if (parser.isSet(shutdownOption)){
        leaveDialog->setConfirmText(LeaveWidget::Shutdown);
    }else if (parser.isSet(rebootOption)){
        leaveDialog->setConfirmText(LeaveWidget::Reboot);
    }
    else{
        qWarning() << parser.helpText();
        return 0;
    }
    int ret =  a.exec();
    delete leaveDialog;
    return ret;
}


